@@ -2,13 +2,11 @@ from flopt.env import Environment
 env = Environment()
 
 from flopt.variable import Variable
-from flopt.expression import CustomExpression
+from flopt.expression import CustomExpression, Const
 from flopt.problem import Problem
 from flopt.solvers import Solver, Solver_list, allAvailableSolvers
 from flopt.solution import Solution
+from flopt.utils import Sum, Prod, Dot, Value
 
 # performance
 import flopt.performance
-
-# specific problems
-from flopt.problems import LpProblem
