@@ -1,18 +1,25 @@
 
 class BaseDataset:
-    """
-    Base Dataset
+    """Base Dataset
     """
     def create_instance(self, instance_name):
-        """
-        defined each dataset
+        """defined each dataset
+
+        Parameters
+        ----------
+        instance_name : str
+
+        Returns
+        -------
+        BaseInstance
 
         .. note::
 
           The formulation is changed by algorithm of solver
 
         """
-        pass
+        raise NotImplementedError()
+
 
     def genInstances(self):
         """
@@ -21,22 +28,38 @@ class BaseDataset:
         for instance_name in self.instance_names:
             yield self.createInstance(instance_name)
 
+
     def createProblem(self, solver):
         """
         Create problem according to solver
         """
         return None
 
+
     def __iter__(self):
         return self.genInstances()
 
 
+
 class BaseInstance:
+    """Base Instance
     """
-    Base Instance
-    """
+    def createProblem(self, solver):
+        """create probelm
+
+        Parameters
+        ----------
+        solver : Solver
+
+        Returns
+        -------
+        (bool, Problem)
+          if solver can be solve this instance return
+        """
+        raise NotImplementedError()
+
+
     def getBestValue(self):
+        """return the optimal value of objective function
         """
-        return the optimal value of objective function
-        """
-        return None
+        raise NotImplementedError()
