@@ -1,4 +1,3 @@
-from copy import deepcopy
 from .base_dataset import BaseDataset, BaseInstance
 
 class CustomDataset(BaseDataset):
@@ -78,19 +77,23 @@ class CustomDataset(BaseDataset):
         self.instance_names = [prob.name for prob in probs]
         self.instance_dict = {prob.name: prob for prob in probs}
 
+
     def createInstance(self, instance_name):
         prob = self.instance_dict[instance_name]
-        return CustomInstance(deepcopy(prob))
+        return CustomInstance(prob)
+
 
     def addProblem(self, prob):
         self.instance_names.append(prob.name)
         self.instance_dict[prob.name] = prob
 
+
     def __iadd__(self, prob):
         self.addProblem(prob)
         return self
 
 
+
 class CustomInstance(BaseInstance):
     """
     Custom Instance
@@ -108,9 +111,20 @@ class CustomInstance(BaseInstance):
         self.name = prob.name
         self.prob = prob
         self.prob_type = prob.prob_type
+        self.var_values = {var.name: var.value() for var in prob.getVariables()}
+
 
     def createProblem(self, solver):
         if set(self.prob_type) & set(solver.can_solve_problems):
-            return True, deepcopy(self.prob)
+            for variable in self.prob.getVariables():
+                variable.setValue(self.var_values[variable.name])
+            return True, self.prob
         else:
             return False, None
+
+
+    def getBestValue(self):
+        """return the optimal value of objective function
+        """
+        return None
+
