@@ -1,5 +1,5 @@
-import inspect
-from time import time
+import time
+
 import flopt
 from flopt.solvers.solver_utils import (
     Log, start_solver_message,
@@ -8,20 +8,20 @@ from flopt.solvers.solver_utils import (
     end_solver_message,
 )
 from flopt.env import setup_logger
-import flopt.constants
+from flopt.constants import SolverTerminateState
+import flopt.error
 
 
 logger = setup_logger(__name__)
 
 
 class BaseSearch:
-    """
-    Base Search Class
+    """Base Search Class
 
     For developer;
 
     - `self.best_solution` has references to variables defined by the user
-    - `self.obj.value(solution)` returns the objective value by the solution
+    - `self.getObjValue(solution)` returns the objective value by the solution
     - `self.recordLog()` records the log (objective value, time, iteratino)
       for each time incumbent solution (`self.bset_solution`) is updated.
 
@@ -31,7 +31,7 @@ class BaseSearch:
         name of solver
     feasible_guard : str
         type of guarder to keep feasibility of solution
-    can_solve_problems : list of str
+    can_solve_problems : list of {'blackbox', 'lp', 'qp', 'permutation'}
         problem names can be solved by this solver
     best_solution : Solution
         best solution
@@ -47,6 +47,8 @@ class BaseSearch:
         type of guarder to keep feasibility of solution
     timelimit : float
         timelimit, unit is second
+    lowerbound : float
+        solver terminates when it obtains the solution whose objective value is lower than this
     msg : bool
         if true, then display logs
     callbacks : list of function
@@ -75,9 +77,10 @@ class BaseSearch:
         self.best_obj_value = float('inf')
         self.best_bd = None
         self.solution = None
-        self.obj = None
         # parameters
         self.timelimit = float('inf')
+        self.lowerbound = -float('inf')
+        self.tol = 1e-10
         self.msg = False
         self.callbacks = []
         # for log
@@ -89,8 +92,7 @@ class BaseSearch:
 
 
     def setParams(self, params=None, feasible_guard=None, **kwargs):
-        """
-        set some parameters
+        """set some parameters
 
         Parameters
         ----------
@@ -119,18 +121,13 @@ class BaseSearch:
         self.save_solution = False
 
 
-    def solve(self, solution, obj, constraints, prob=None, msg=False):
-        """
-        solve the problem of (solution, obj)
+    def solve(self, solution, prob, msg=False):
+        """solve the problem of (solution, obj)
 
         Parameters
         ----------
         solution : Solution
             solution object
-        obj : expression or VarElement family
-            objective function
-        constraints : list of Constraint
-            constraints
         prob : Problem
             problem
         msg : bool
@@ -140,87 +137,112 @@ class BaseSearch:
         -------
         status, Log
         """
-        if prob is not None and not self.available(prob):
-            logger.error(f'problem can not be solved by solver {self.name}')
-            status = flopt.constants.SOLVER_ABNORMAL_TERMINATE
-            raise flopt.constants.SolverError
+        if not self.available(prob, verbose=True):
+            logger.error(f'Problem can not be solved by solver {self.name}.')
+            status = SolverTerminateState.Abnormal
+            raise flopt.error.SolverError
 
-        self.best_solution = solution
         self.solution = solution.clone()
         self.prob = prob
-        self.obj = obj
-        self.constraints = constraints
-        self.start_time = time()
         self.msg = msg
+        self.best_solution = solution
 
+        self.start_time = time.time()
         if msg:
             params = {'timelimit': self.timelimit}
             start_solver_message(self.name, params, solution)
 
         try:
+            self.startProcess()
             status = self.search()
+            self.closeProcess()
+        except flopt.error.RearchLowerbound:
+            status = SolverTerminateState.Lowerbound
         except KeyboardInterrupt:
             print('Get user ctrl-cuser ctrl-c')
-            status = flopt.constants.SOLVER_INTERRUPT_TERMINATE
+            status = SolverTerminateState.Interrupt
 
         if msg:
-            obj_value = self.obj.value(self.best_solution)
-            end_solver_message(status, obj_value, time()-self.start_time)
+            obj_value = self.prob.obj.value(self.best_solution)
+            end_solver_message(status, obj_value, time.time()-self.start_time)
 
-        return status, self.log, time()-self.start_time
+        return status, self.log, time.time()-self.start_time
 
 
     def updateSolution(self, solution, obj_value=None):
-        """
-        update self.best_solution
+        """update self.best_solution
         """
         self.best_solution.copy(solution)
         if obj_value is None:
-            self.best_obj_value = self.obj.value(solution)
+            self.best_obj_value = self.prob.obj.value(solution)
         else:
             self.best_obj_value = obj_value
             self.save_solution = True
 
 
     def recordLog(self):
-        """
-        write log in `self.log`
+        """write log in `self.log`
         """
         log_dict = {
             'obj_value': self.best_obj_value,
             'best_bd': self.best_bd,
-            'time': time()-self.start_time,
+            'time': time.time()-self.start_time,
             'iteration': self.trial_ix
         }
         if self.max_k > 1 and self.save_solution:
             log_dict['solution'] = self.best_solution.clone()
             self.save_solution = False
         self.log.append(log_dict)
+        if self.best_obj_value < self.lowerbound + self.tol:
+            if self.msg:
+                self.during_solver_message('*')
+            raise flopt.error.RearchLowerbound()
 
 
     def during_solver_message(self, head):
         during_solver_message(head, self.best_obj_value,
-            self.best_bd, time()-self.start_time, self.trial_ix)
+            self.best_bd, time.time()-self.start_time, self.trial_ix)
 
 
     def search(self):
-        raise NotImplementedError
+        raise NotImplementedError()
 
 
     def available(self, prob):
-        raise NotImplementedError
+        raise NotImplementedError()
 
 
-    def __str__(self):
-        s  = f'Name: Solver\n'
-        s += f'  Type : {self.name}\n'
-        attrobjs = [
-            (attr, obj) for (attr, obj) in inspect.getmembers(self)
-            if not callable(obj) and attr[:2] != "__" and attr[-2:] != "__" and attr != 'name'
-        ]
-        for attr, obj in attrobjs:
-            s += f'  {attr} : {obj}\n'
-        s += '\n'
-        return s
+    def getObjValue(self, solution):
+        """calculate objective value
+
+        Parameters
+        ----------
+        solution : Solution
+
+        Returns
+        -------
+        int or float
+        """
+        return self.prob.obj.value(solution)
 
 
+    def startProcess(self):
+        """process of beginning of search
+        """
+        if all(const.feasible(self.best_solution) for const in self.prob.constraints):
+            self.best_obj_value = self.prob.obj.value(self.best_solution)
+        else:
+            self.best_obj_value = float('inf')
+        self.recordLog()
+        if self.best_obj_value < self.lowerbound + self.tol:
+            raise flopt.error.RearchLowerbound()
+
+        if self.msg:
+            during_solver_message_header()
+            self.during_solver_message('S')
+
+
+    def closeProcess(self):
+        """process of ending of search
+        """
+        self.recordLog()
