@@ -2,13 +2,10 @@ from time import time
 
 from flopt.solvers.base import BaseSearch
 from flopt.solvers.solver_utils import (
-    Log, start_solver_message,
-    during_solver_message_header,
     during_solver_message,
-    end_solver_message
 )
 from flopt.env import setup_logger
-import flopt.constants
+from flopt.constants import SolverTerminateState
 
 
 logger = setup_logger(__name__)
@@ -45,19 +42,17 @@ class SequentialUpdateSearch(BaseSearch):
         search a better solution using `self.setNewSolution()` function
         `self.setNewSolution()` generate new solution and set it into self.solution
         """
-        self.startProcess()
-        status = flopt.constants.SOLVER_NORMAL_TERMINATE
+        status = SolverTerminateState.Normal
 
         for self.trial_ix in range(1, int(self.n_trial)+1):
             # check time limit
             if time() > self.start_time + self.timelimit:
-                self.closeProcess()
-                status = flopt.constants.SOLVER_TIMELIMIT_TERMINATE
+                status = SolverTerminateState.Timelimit
                 return status
 
             # generate new solution and set it into self.solution
             self.setNewSolution()
-            obj_value = self.obj.value(self.solution)
+            obj_value = self.getObjValue(self.solution)
 
             # check whether update or not
             if obj_value < self.best_obj_value:
@@ -70,27 +65,9 @@ class SequentialUpdateSearch(BaseSearch):
             for callback in self.callbacks:
                 callback([self.solution], self.best_solution, self.best_obj_value)
 
-        self.closeProcess()
         return status
 
 
     def setNewSolution(self):
         raise NotImplementedError()
 
-
-    def startProcess(self):
-        """
-        set initial value to self variables and objective value
-        , and display start log
-        """
-        self.best_obj_value = self.obj.value(self.best_solution)
-        self.recordLog()
-
-        if self.msg:
-            during_solver_message_header()
-            self.during_solver_message('S')
-
-
-    def closeProcess(self):
-        self.recordLog()
-
