@@ -1,5 +1,6 @@
 import random
 from .base_sequential_update import SequentialUpdateSearch
+from flopt.constants import VariableType
 
 class TwoOpt(SequentialUpdateSearch):
     """2-Opt: a kind of local search for permutation.
@@ -16,21 +17,28 @@ class TwoOpt(SequentialUpdateSearch):
         self.can_solve_problems = ['permutation']
 
 
-    def available(self, prob):
+    def available(self, prob, verbose=False):
         """
         Parameters
         ----------
-        obj : Expression or VarElement family
-            objective function
-        constraints : list of Constraint
-            constraints
+        prob : Problem
+        verbose : bool
 
         Returns
         -------
         bool
             return true if it can solve the problem else false
         """
-        return all(var.getType() == 'VarPermutation' for var in prob.getVariables())
+        for var in prob.getVariables():
+            if not var.type() == VariableType.Permutation:
+                if verbose:
+                    logger.error(f"variable: \n{var}\n must be permutation, but got {var.type()}")
+                return False
+        if prob.constraints:
+            if verbose:
+                logger.error(f"this solver can not handle constraints")
+            return False
+        return True
 
 
     def setNewSolution(self, *args, **kwargs):
@@ -45,7 +53,7 @@ class TwoOpt(SequentialUpdateSearch):
             # 2-opt
             new_perm = perm[:i] + perm[i:j][::-1] + perm[j:]
             var.setValue(new_perm)
-            if self.obj.value(self.solution) >= self.best_obj_value:
+            if self.getObjValue(self.solution) >= self.best_obj_value:
                 var.setValue(perm)
 
 
