@@ -4,11 +4,11 @@ from flopt import Variable, CustomExpression
 
 @pytest.fixture(scope='function')
 def a():
-    return Variable('a', lowBound=1, upBound=3, iniValue=2, cat='Integer')
+    return Variable('a', lowBound=1, upBound=3, ini_value=2, cat='Integer')
 
 @pytest.fixture(scope='function')
 def b():
-    return Variable('b', lowBound=1, upBound=3, iniValue=2, cat='Continuous')
+    return Variable('b', lowBound=1, upBound=3, ini_value=2, cat='Continuous')
 
 @pytest.fixture(scope='function')
 def obj():
@@ -99,6 +99,9 @@ def test_CustomExpression_getVariable_Variable(custom_obj, a, b):
     assert (custom_obj+a).getVariables() == {a, b}
     assert (custom_obj+(a+b)).getVariables() == {a,b}
 
+def test_CustomExpression_traverse(custom_obj):
+    assert len(list(custom_obj.traverse())) == 1
+
 def test_CustomExpression_neg(custom_obj):
     assert (-custom_obj).value() == -6
 
@@ -112,11 +115,10 @@ def test_CustomExpression_abs(custom_obj):
 def test_CustomExpression_hash(custom_obj):
     hash(custom_obj)
 
-def test_CustomExpression_hasCustomExpression(custom_obj, a):
-    assert custom_obj.hasCustomExpression() == True
-    assert (custom_obj + a).hasCustomExpression() == True
+def test_CustomExpression_repr(custom_obj):
+    repr(custom_obj)
 
 def test_CustomExpression_isLinear(custom_obj, a):
-    assert (custom_obj).isLinear() == 'Unknown'
-    assert (custom_obj+a).isLinear() == 'Unknown'
+    assert (custom_obj).isLinear() == False
+    assert (custom_obj+a).isLinear() == False
 
