@@ -1,15 +1,15 @@
 import pytest
 
 from flopt import Variable
-from flopt.expression import ExpressionConst
+from flopt.expression import Const
 
 @pytest.fixture(scope='function')
 def a():
-    return Variable(name='a', lowBound=1, upBound=5, iniValue=2, cat='Continuous')
+    return Variable(name='a', lowBound=1, upBound=5, ini_value=2, cat='Continuous')
 
 @pytest.fixture(scope='function')
 def b():
-    return Variable(name='b', lowBound=1, upBound=5, iniValue=3, cat='Continuous')
+    return Variable(name='b', lowBound=1, upBound=5, ini_value=3, cat='Continuous')
 
 @pytest.fixture(scope='function')
 def c(a, b):
@@ -61,6 +61,46 @@ def test_Expression_pow(c, a):
 def test_Expression_getVariable(a, b, c):
     assert c.getVariables() == {a, b}
 
+def test_Expression_constant1(a):
+    assert (a+1).constant() == 1
+
+def test_Expression_constant2(a, b):
+    assert (a/b+1).constant() == 1
+
+def test_Expression_isMonomial1(a, b):
+    assert (2*a).isMonomial() == True
+    assert (2*a*b).isMonomial() == True
+    assert (a+1).isMonomial() == False
+    assert (a/b).isMonomial() == False
+
+def test_Expression_isMonomial2(a, b):
+    (2*a).toMonomial()
+    (2*a*b).toMonomial()
+
+def test_Expression_simplify(a, b):
+    (a+1-a).simplify()
+    (a*a+a*b+a).simplify()
+
+def test_Expression_expand(a, b):
+    (a+1-a).expand()
+    (a*a+a*b+a).expand()
+
+def test_Expression_toBinary():
+    b = Variable('bb', cat='Binary')
+    s = Variable('ss', cat='Spin')
+    i = Variable('ii', lowBound=-1, upBound=1, cat='Integer')
+    print((b+1).toBinary())
+    print((s+1).toBinary())
+    print((i+1).toBinary())
+
+def test_Expression_toSpin():
+    b = Variable('bb', cat='Binary')
+    s = Variable('ss', cat='Spin')
+    i = Variable('ii', lowBound=-1, upBound=1, cat='Integer')
+    print((b+1).toSpin())
+    print((s+1).toSpin())
+    print((i+1).toSpin())
+
 def test_Expression_neg(c):
     assert (-c).value() == -5
 
@@ -75,35 +115,19 @@ def test_Expression_cas(c):
     assert isinstance(int(c), int)
     assert isinstance(float(c), float)
 
-def test_Expression_hashc(c):
+def test_Expression_hash(c):
     hash(c)
 
-def test_Expression_hasCustomExpression(c, a):
-    assert a.hasCustomExpression() == False
-    assert c.hasCustomExpression() == False
-
 def test_Expression_isLinear(a, b, c):
     assert a.isLinear() == True
     assert c.isLinear() == True
     assert (a*b).isLinear() == False
 
+def test_Const_hash():
+    hash(Const(0))
 
-def test_ExpressionConst_hash():
-    hash(ExpressionConst(0))
-
-def test_ExpressionConst_hasCustomExpression():
-    assert ExpressionConst(0).hasCustomExpression() == False
-
-def test_ExpressionConst_isLinear():
-    assert ExpressionConst(0).isLinear() == True
-
-
-def test_Expression_maxDegree(a, b, c):
-    assert a.maxDegree() == 1
-    assert (a*a).maxDegree() == 2
-    assert (a*c).maxDegree() == 2   # a*(a*b)
-    assert (a*a*b*c).maxDegree() == 3  # a*a*(a+b)
-
+def test_Const_isLinear():
+    assert Const(0).isLinear() == True
 
 def test_Expression_isIsing(a, b):
     import numpy as np
@@ -116,10 +140,6 @@ def test_Expression_isIsing(a, b):
     obj = - (x.T).dot(J).dot(x) - (h.T).dot(x)
     assert obj.isIsing()
 
-    obj += a*a*a
-    assert not obj.isIsing()
-
-
 def test_Expression_toIsing(a, b):
     import numpy as np
     x = np.array([Variable('a', cat='Spin'), Variable('b', cat='Spin')])
