@@ -4,10 +4,16 @@ import pytest
 import numpy as np
 
 from flopt import Variable
+from flopt.constants import VariableType
+
+
+@pytest.fixture(scope='function')
+def a():
+    return Variable('a', lowBound=1, upBound=3, ini_value=2, cat='Continuous')
 
 @pytest.fixture(scope='function')
 def b():
-    return Variable('b', lowBound=1, upBound=3, iniValue=2, cat='Continuous')
+    return Variable('b', lowBound=1, upBound=3, ini_value=2, cat='Continuous')
 
 # add, sub, mul and div
 def test_VarContinuous_add(b):
@@ -26,7 +32,7 @@ def test_VarContinuous_sub(b):
     assert math.isclose((b-np.float64(2.1)).value(), -0.1)
     assert math.isclose((np.float64(2.1)-b).value(), 0.1)
 
-def test_VarContinuous_mul(b):
+def test_VarContinuous_mul1(b):
     assert (b*2).value() == 4
     assert (2*b).value() == 4
     assert (b*2.1).value() == 4.2
@@ -34,6 +40,10 @@ def test_VarContinuous_mul(b):
     assert (b*np.float64(2.1)).value() == 4.2
     assert (np.float64(2.1)*b).value() == 4.2
 
+def test_VarBinary_mul2(a, b):
+    assert ((-a)*b).name == (a*(-b)).name
+    assert (a*b).name == ((-a)*(-b)).name
+
 def test_VarContinuous_div(b):
     assert (b/2).value() == 1
     assert (2/b).value() == 1
@@ -65,8 +75,8 @@ def test_VarInteger_hash(b):
     hash(b)
 
 # base function
-def test_VarContinuous_getType(b):
-    assert b.getType() == 'VarContinuous'
+def test_VarContinuous_type(b):
+    assert b.type() == VariableType.Continuous
 
 def test_VarContinuous_getvariable(b):
     assert b.getVariables() == {b}
@@ -85,5 +95,5 @@ def test_VarContinuous_colne(b):
     assert _b.value() == b.value()
     assert _b.getLb() == b.getLb()
     assert _b.getUb() == b.getUb()
-    assert _b.getType() == b.getType()
+    assert _b.type() == b.type()
 
