@@ -4,10 +4,16 @@ import pytest
 import numpy as np
 
 from flopt import Variable
+from flopt.constants import VariableType
 
 @pytest.fixture(scope='function')
 def a():
-    return Variable('a', lowBound=1, upBound=3, iniValue=2, cat='Integer')
+    return Variable('a', lowBound=1, upBound=3, ini_value=2, cat='Integer')
+
+@pytest.fixture(scope='function')
+def b():
+    return Variable('b', lowBound=1, upBound=3, ini_value=2, cat='Integer')
+
 
 # add, sub, mul, div and pow
 def test_VarInteger_add(a):
@@ -26,7 +32,7 @@ def test_VarInteger_sub(a):
     assert math.isclose((a-np.float64(2.1)).value(), -0.1)
     assert math.isclose((np.float64(2.1)-a).value(), 0.1)
 
-def test_VarInteger_mul(a):
+def test_VarInteger_mul1(a):
     assert (a*2).value() == 4
     assert (2*a).value() == 4
     assert (a*2.1).value() == 4.2
@@ -34,6 +40,10 @@ def test_VarInteger_mul(a):
     assert (a*np.float64(2.1)).value() == 4.2
     assert (np.float64(2.1)*a).value() == 4.2
 
+def test_VarBinary_mul2(a, b):
+    assert ((-a)*b).name == (a*(-b)).name
+    assert (a*b).name == ((-a)*(-b)).name
+
 def test_VarInteger_div(a):
     assert (a/2).value() == 1
     assert (1/a).value() == 0.5
@@ -65,8 +75,8 @@ def test_VarInteger_hash(a):
     hash(a)
 
 # base function
-def test_VarInteger_getType(a):
-    assert a.getType() == 'VarInteger'
+def test_VarInteger_type(a):
+    assert a.type() == VariableType.Integer
 
 def test_VarInteger_getVariable(a):
     assert a.getVariables() == {a}
@@ -85,5 +95,5 @@ def test_VarInteger_colne(a):
     assert _a.value() == a.value()
     assert _a.getLb() == a.getLb()
     assert _a.getUb() == a.getUb()
-    assert _a.getType() == a.getType()
+    assert _a.type() == a.type()
 
