@@ -1,6 +1,7 @@
 import pytest
 
 from flopt import Variable
+from flopt.constants import VariableType
 
 @pytest.fixture(scope='function')
 def a():
@@ -10,8 +11,8 @@ def test_VarPermutation_hash(a):
     hash(a)
 
 # base function
-def test_VarPermutation_getType(a):
-    assert a.getType() == 'VarPermutation'
+def test_VarPermutation_type(a):
+    assert a.type() == VariableType.Permutation
 
 def test_VarPermutation_getVariable(a):
     assert a.getVariables() == {a}
@@ -23,5 +24,5 @@ def test_VarPermutation_colne(a):
     assert _a.value() == a.value()
     assert _a.getLb() == a.getLb()
     assert _a.getUb() == a.getUb()
-    assert _a.getType() == a.getType()
+    assert _a.type() == a.type()
 
